# -*- coding: utf-8 -*-

"""
***************************************************************************
    r_li_renyi_ascii.py
    -------------------
    Date                 : February 2016
    Copyright            : (C) 2016 by Médéric Ribreux
    Email                : medspx at medspx dot fr
***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************
"""

__author__ = 'Médéric Ribreux'
__date__ = 'February 2016'
__copyright__ = '(C) 2016, Médéric Ribreux'

# This will get replaced with a git SHA1 when you do a git archive

__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'

from .r_li import checkMovingWindow, configFile, moveOutputTxtFile


def checkParameterValuesBeforeExecuting(alg, parameters, context):
    return checkMovingWindow(alg, parameters, context, True)


def processCommand(alg, parameters, context, feedback):
    configFile(alg, parameters, context, feedback, True)


def processOutputs(alg, parameters, context, feedback):
    moveOutputTxtFile(alg, parameters, context)
